@@ -5,10 +5,13 @@ This module contains some of the most central FireWorks classes:
 
 
 - A Workflow is a sequence of FireWorks as a DAG (directed acyclic graph)
-- A FireWork defines a workflow step and contains one or more FireTasks along with its Launches.
+- A FireWork defines a workflow step and contains one or more FireTasks along
+ with its Launches.
 - A Launch describes the run of a FireWork on a computing resource.
-- A FireTaskBase defines the contract for tasks that run within a FireWork (FireTasks)
-- A FWAction encapsulates the output of a FireTask and tells FireWorks what to do next after a job completes
+- A FireTaskBase defines the contract for tasks that run within a FireWork (
+FireTasks)
+- A FWAction encapsulates the output of a FireTask and tells FireWorks what
+to do next after a job completes
 """
 
 from collections import defaultdict
@@ -16,7 +19,8 @@ from collections import defaultdict
 import datetime
 from fireworks.core.fworker import FWorker
 from fireworks.utilities.dict_mods import apply_mod
-from fireworks.utilities.fw_serializers import FWSerializable, recursive_serialize, recursive_deserialize, serialize_fw
+from fireworks.utilities.fw_serializers import FWSerializable, \
+    recursive_serialize, recursive_deserialize, serialize_fw
 from fireworks.utilities.fw_utilities import get_my_host, get_my_ip
 
 __author__ = "Anubhav Jain"
@@ -29,7 +33,8 @@ __date__ = "Feb 5, 2013"
 
 class FireTaskBase(dict, FWSerializable):
     """
-    FireTaskBase is used like an abstract class that defines a computing task (FireTask). All FireTasks
+    FireTaskBase is used like an abstract class that defines a computing task
+     (FireTask). All FireTasks
     should inherit from FireTaskBase.
 
     You can set parameters of a FireTask like you'd use a dict.
@@ -37,8 +42,10 @@ class FireTaskBase(dict, FWSerializable):
 
     def run_task(self, fw_spec):
         """
-        This method gets called when the FireTask is run. It can take in a FireWork spec,
-        perform some task using that data, and then return an output in the form of a FWAction.
+        This method gets called when the FireTask is run. It can take in a
+        FireWork spec,
+        perform some task using that data, and then return an output in the
+        form of a FWAction.
 
         :param fw_spec: (dict) a FireWork spec
         :return: (FWAction)
@@ -57,7 +64,8 @@ class FireTaskBase(dict, FWSerializable):
 
     def __getitem__(self, key):
         """
-        Reproduce a simple defaultdict-like behavior - any unset parameters return None
+        Reproduce a simple defaultdict-like behavior - any unset parameters
+        return None
         """
         try:
             return dict.__getitem__(self, key)
@@ -67,20 +75,28 @@ class FireTaskBase(dict, FWSerializable):
 
 class FWAction(FWSerializable):
     """
-    A FWAction encapsulates the output of a FireTask (it is returned by a FireTask after the FireTask completes). The
-     FWAction allows a user to store rudimentary output data as well as return commands that alter the workflow.
+    A FWAction encapsulates the output of a FireTask (it is returned by a
+    FireTask after the FireTask completes). The
+     FWAction allows a user to store rudimentary output data as well as
+     return commands that alter the workflow.
     """
 
-    def __init__(self, stored_data=None, exit=False, update_spec=None, mod_spec=None, additions=None, detours=None,
+    def __init__(self, stored_data=None, exit=False, update_spec=None,
+                 mod_spec=None, additions=None, detours=None,
                  defuse_children=False):
         """
-        :param stored_data: (dict) data to store from the run. Does not affect the operation of FireWorks.
-        :param exit: (bool) if set to True, any remaining FireTasks within the same FireWork are skipped.
+        :param stored_data: (dict) data to store from the run. Does not
+        affect the operation of FireWorks.
+        :param exit: (bool) if set to True, any remaining FireTasks within
+        the same FireWork are skipped.
         :param update_spec: (dict) specifies how to update the child FW's spec
-        :param mod_spec: ([dict]) update the child FW's spec using the DictMod language (more flexible than update_spec)
+        :param mod_spec: ([dict]) update the child FW's spec using the
+        DictMod language (more flexible than update_spec)
         :param additions: ([Workflow]) a list of WFs/FWs to add as children
-        :param detours: ([Workflow]) a list of WFs/FWs to add as children (they will inherit the current FW's children)
-        :param defuse_children: (bool) defuse all the original children of this FireWork
+        :param detours: ([Workflow]) a list of WFs/FWs to add as children (
+        they will inherit the current FW's children)
+        :param defuse_children: (bool) defuse all the original children of
+        this FireWork
         """
         mod_spec = mod_spec if mod_spec is not None else []
         additions = additions if additions is not None else []
@@ -90,14 +106,17 @@ class FWAction(FWSerializable):
         self.exit = exit
         self.update_spec = update_spec if update_spec else {}
         self.mod_spec = mod_spec if isinstance(mod_spec, list) else [mod_spec]
-        self.additions = additions if isinstance(additions, list) else [additions]
+        self.additions = additions if isinstance(additions, list) else [
+            additions]
         self.detours = detours if isinstance(detours, list) else [detours]
         self.defuse_children = defuse_children
 
     @recursive_serialize
     def to_dict(self):
-        return {'stored_data': self.stored_data, 'exit': self.exit, 'update_spec': self.update_spec,
-                'mod_spec': self.mod_spec, 'additions': self.additions, 'detours': self.detours,
+        return {'stored_data': self.stored_data, 'exit': self.exit,
+                'update_spec': self.update_spec,
+                'mod_spec': self.mod_spec, 'additions': self.additions,
+                'detours': self.detours,
                 'defuse_children': self.defuse_children}
 
     @classmethod
@@ -106,17 +125,20 @@ class FWAction(FWSerializable):
         d = m_dict
         additions = [Workflow.from_dict(f) for f in d['additions']]
         detours = [Workflow.from_dict(f) for f in d['detours']]
-        return FWAction(d['stored_data'], d['exit'], d['update_spec'], d['mod_spec'], additions, detours,
+        return FWAction(d['stored_data'], d['exit'], d['update_spec'],
+                        d['mod_spec'], additions, detours,
                         d['defuse_children'])
 
     @property
     def skip_remaining_tasks(self):
         """
-        If the FWAction gives any dynamic action, we skip the subsequent FireTasks
+        If the FWAction gives any dynamic action, we skip the subsequent
+        FireTasks
 
         :return: (bool)
         """
-        return self.exit or self.update_spec or self.mod_spec or self.detours or self.additions or self.defuse_children
+        return self.exit or self.update_spec or self.mod_spec or self.detours \
+                   or self.additions or self.defuse_children
 
 
 class FireWork(FWSerializable):
@@ -124,18 +146,23 @@ class FireWork(FWSerializable):
     A FireWork is a workflow step and might be contain several FireTasks
     """
 
-    # 'Canceled' is the dominant spelling over 'cancelled' in the US starting around 1985...(Google n-grams)
-    STATE_RANKS = {'DEFUSED': 0, 'WAITING': 1, 'READY': 2, 'RESERVED': 3, 'FIZZLED': 4, 'RUNNING': 5, 'CANCELED': 6,
+    # 'Canceled' is the dominant spelling over 'cancelled' in the US starting
+    #  around 1985...(Google n-grams)
+    STATE_RANKS = {'DEFUSED': 0, 'WAITING': 1, 'READY': 2, 'RESERVED': 3,
+                   'FIZZLED': 4, 'RUNNING': 5, 'CANCELED': 6,
                    'COMPLETED': 7}
 
-    def __init__(self, tasks, spec=None, launches=None, archived_launches=None, state='WAITING', created_on=None,
-                 fw_id=-1):
+    def __init__(self, tasks, spec=None, name=None, launches=None, archived_launches=None,
+                 state='WAITING', created_on=None, fw_id=-1):
         """
         :param tasks: ([FireTask]) a list of FireTasks to run in sequence
-        :param spec: (dict) specification of the job to run. Used by the FireTask
+        :param spec: (dict) specification of the job to run. Used by the
+        FireTask
         :param launches: ([Launch]) a list of Launch objects of this FireWork
-        :param archived_launches: ([Launch]) a list of archived Launch objects of this FireWork
-        :param state: (str) the state of the FW (e.g. WAITING, RUNNING, COMPLETED, CANCELED)
+        :param archived_launches: ([Launch]) a list of archived Launch
+        objects of this FireWork
+        :param state: (str) the state of the FW (e.g. WAITING, RUNNING,
+        COMPLETED, CANCELED)
         :param fw_id: (int) an identification number for this FireWork
         """
 
@@ -143,16 +170,22 @@ class FireWork(FWSerializable):
 
         self.tasks = tasks
         self.spec = spec if spec else {}
-        self.spec['_tasks'] = [t.to_dict() for t in tasks]  # put tasks in a special location of the spec
+        self.spec['_tasks'] = [t.to_dict() for t in
+                               tasks]  # put tasks in a special location of the spec
+
+        self.name = name if name else 'Unnamed FW'  # do it this way to prevent None names
         self.fw_id = fw_id
         self.launches = launches if launches else []
         self.archived_launches = archived_launches if archived_launches else []
-        self.created_on = created_on if created_on else datetime.datetime.utcnow()
+        self.created_on = created_on if created_on else datetime.datetime \
+            .utcnow()
+
         self.state = state
 
     @recursive_serialize
     def to_dict(self):
-        m_dict = {'spec': self.spec, 'fw_id': self.fw_id, 'created_on': self.created_on}
+        m_dict = {'spec': self.spec, 'fw_id': self.fw_id,
+                  'created_on': self.created_on}
 
         # only serialize these fields if non-empty
         if len(self.launches) > 0:
@@ -164,12 +197,16 @@ class FireWork(FWSerializable):
         if self.state != 'WAITING':
             m_dict['state'] = self.state
 
+        m_dict['name'] = self.name
+
         return m_dict
 
     def _rerun(self):
         """
-        Moves all Launches to archived Launches and resets the state to 'WAITING'. The FireWork can thus be re-run \
-        even if it was Launched in the past. This method should be called by a Workflow because a refresh is needed \
+        Moves all Launches to archived Launches and resets the state to
+        'WAITING'. The FireWork can thus be re-run \
+        even if it was Launched in the past. This method should be called by
+        a Workflow because a refresh is needed \
         after calling this method.
 
         """
@@ -180,9 +217,12 @@ class FireWork(FWSerializable):
 
     def to_db_dict(self):
         m_dict = self.to_dict()
-        m_dict['launches'] = [l.launch_id for l in self.launches]  # the launches are stored separately
+        m_dict['launches'] = [l.launch_id for l in
+                              self.launches]  # the launches are stored
+        # separately
         m_dict['archived_launches'] = [l.launch_id for l in
-                                       self.archived_launches]  # the archived launches are stored separately
+                                       self.archived_launches]  # the
+        # archived launches are stored separately
         m_dict['state'] = self.state
         return m_dict
 
@@ -191,29 +231,37 @@ class FireWork(FWSerializable):
     def from_dict(cls, m_dict):
         tasks = m_dict['spec']['_tasks']
         launches = [Launch.from_dict(tmp) for tmp in m_dict.get('launches', [])]
-        archived_launches = [Launch.from_dict(tmp) for tmp in m_dict.get('archived_launches', [])]
+        archived_launches = [Launch.from_dict(tmp) for tmp in
+                             m_dict.get('archived_launches', [])]
         fw_id = m_dict.get('fw_id', -1)
         state = m_dict.get('state', 'WAITING')
         created_on = m_dict.get('created_on', None)
+        name = m_dict.get('name', None)
 
-        return FireWork(tasks, m_dict['spec'], launches, archived_launches, state, created_on, fw_id)
+        return FireWork(tasks, m_dict['spec'], name, launches, archived_launches,
+                        state, created_on, fw_id)
 
 
 class Launch(FWSerializable, object):
     """
-    A Launch encapsulates data about a specific run of a FireWork on a computing resource
+    A Launch encapsulates data about a specific run of a FireWork on a
+    computing resource
     """
 
-    def __init__(self, state, launch_dir, fworker=None, host=None, ip=None, action=None, state_history=None,
+    def __init__(self, state, launch_dir, fworker=None, host=None, ip=None,
+                 action=None, state_history=None,
                  launch_id=None, fw_id=None):
         """
         :param state: (str) the state of the Launch (e.g. RUNNING, COMPLETED)
         :param launch_dir: (str) the directory where the Launch takes place
         :param fworker: (FWorker) The FireWorker running the Launch
-        :param host: (str) the hostname where the launch took place (set automatically if None)
-        :param ip: (str) the IP address where the launch took place (set automatically if None)
+        :param host: (str) the hostname where the launch took place (set
+        automatically if None)
+        :param ip: (str) the IP address where the launch took place (set
+        automatically if None)
         :param action: (FWAction) the output of the Launch
-        :param state_history: ([dict]) a history of all states of the Launch and when they occurred
+        :param state_history: ([dict]) a history of all states of the Launch
+        and when they occurred
         :param launch_id: (int) launch_id set by the LaunchPad
         :param fw_id: (int) id of the FireWork this Launch is running
         """
@@ -233,7 +281,8 @@ class Launch(FWSerializable, object):
 
     def touch_history(self):
         """
-        Updates the update_at field of the state history of a Launch. Used to ping that a Launch is still alive.
+        Updates the update_at field of the state history of a Launch. Used to
+         ping that a Launch is still alive.
         """
         self.state_history[-1]['updated_on'] = datetime.datetime.utcnow()
 
@@ -241,7 +290,8 @@ class Launch(FWSerializable, object):
         """
         Adds the job_id to the reservation
 
-        :param reservation_id: (str) the id of the reservation (e.g., queue reservation)
+        :param reservation_id: (str) the id of the reservation (e.g.,
+        queue reservation)
         """
         for data in self.state_history:
             if data['state'] == 'RESERVED' and 'reservation_id' not in data:
@@ -258,7 +308,8 @@ class Launch(FWSerializable, object):
     @state.setter
     def state(self, state):
         """
-        Setter for the the Launch's state. Automatically triggers an update to state_history.
+        Setter for the the Launch's state. Automatically triggers an update
+        to state_history.
 
         :param state: (str) the state to set for the Launch
         """
@@ -289,7 +340,8 @@ class Launch(FWSerializable, object):
     @property
     def last_pinged(self):
         """
-        :return: (datetime) the time the Launch last pinged a heartbeat that it was still running
+        :return: (datetime) the time the Launch last pinged a heartbeat that
+        it was still running
         """
         return self._get_time('RUNNING', True)
 
@@ -306,17 +358,21 @@ class Launch(FWSerializable, object):
     @property
     def reservedtime_secs(self):
         """
-        :return: (int) number of seconds the Launch was stuck as RESERVED in a queue
+        :return: (int) number of seconds the Launch was stuck as RESERVED in
+        a queue
         """
         start = self.time_reserved
         if start:
-            end = self.time_start if self.time_start else datetime.datetime.utcnow()
+            end = self.time_start if self.time_start else datetime.datetime \
+                .utcnow()
             return (end - start).total_seconds()
 
     @recursive_serialize
     def to_dict(self):
-        return {'fworker': self.fworker, 'fw_id': self.fw_id, 'launch_dir': self.launch_dir, 'host': self.host,
-                'ip': self.ip, 'action': self.action, 'state': self.state, 'state_history': self.state_history,
+        return {'fworker': self.fworker, 'fw_id': self.fw_id,
+                'launch_dir': self.launch_dir, 'host': self.host,
+                'ip': self.ip, 'action': self.action, 'state': self.state,
+                'state_history': self.state_history,
                 'launch_id': self.launch_id}
 
     @recursive_serialize
@@ -333,17 +389,22 @@ class Launch(FWSerializable, object):
     @recursive_deserialize
     def from_dict(cls, m_dict):
         fworker = FWorker.from_dict(m_dict['fworker'])
-        action = FWAction.from_dict(m_dict['action']) if m_dict.get('action') else None
-        return Launch(m_dict['state'], m_dict['launch_dir'], fworker, m_dict['host'], m_dict['ip'], action,
-                      m_dict['state_history'], m_dict['launch_id'], m_dict['fw_id'])
+        action = FWAction.from_dict(m_dict['action']) if m_dict.get(
+            'action') else None
+        return Launch(m_dict['state'], m_dict['launch_dir'], fworker,
+                      m_dict['host'], m_dict['ip'], action,
+                      m_dict['state_history'], m_dict['launch_id'],
+                      m_dict['fw_id'])
 
     def _update_state_history(self, state):
         """
-        Internal method to update the state history whenever the Launch state is modified
+        Internal method to update the state history whenever the Launch state
+         is modified
 
         :param state:
         """
-        last_state = self.state_history[-1]['state'] if len(self.state_history) > 0 else None
+        last_state = self.state_history[-1]['state'] if len(
+            self.state_history) > 0 else None
         if state != last_state:
             now_time = datetime.datetime.utcnow()
             self.state_history.append({'state': state, 'created_on': now_time})
@@ -352,10 +413,12 @@ class Launch(FWSerializable, object):
 
     def _get_time(self, states, use_update_time=False):
         """
-        Internal method to help get the time of various events in the Launch (e.g. RUNNING) from the state history
+        Internal method to help get the time of various events in the Launch
+        (e.g. RUNNING) from the state history
 
         :param states: match one of these states
-        :param use_update_time: use the "updated_on" time rather than "created_on"
+        :param use_update_time: use the "updated_on" time rather than
+        "created_on"
         :return: (datetime)
         """
         states = states if isinstance(states, list) else [states]
@@ -376,13 +439,27 @@ class Workflow(FWSerializable):
         An inner class for storing the DAG links between FireWorks
         """
 
+        def __init__(self, *args, **kwargs):
+            super(Workflow.Links, self).__init__(*args, **kwargs)
+
+            for k, v in self.items():
+                if not isinstance(v, list):
+                    self[k] = [v]  # v must be list
+                if not isinstance(k, int):
+                    try:
+                        self[int(k)] = self[k]  # k must be int
+                    except:
+                        pass  # garbage input
+                    del self[k]
+
         @property
         def nodes(self):
             return self.keys()
 
         @property
         def parent_links(self):
-            # note: if performance of parent_links becomes an issue, override delitem/setitem to update parent_links
+            # note: if performance of parent_links becomes an issue,
+            # override delitem/setitem to update parent_links
             child_parents = defaultdict(list)
             for (parent, children) in self.iteritems():
                 for child in children:
@@ -390,30 +467,36 @@ class Workflow(FWSerializable):
             return dict(child_parents)
 
         def to_dict(self):
-            return dict(self)
+            # convert to str form for Mongo, which cannot have int keys
+            return dict([(str(k), v) for (k, v) in self.iteritems()])
 
         def to_db_dict(self):
             # convert to str form for Mongo, which cannot have int keys
-            m_dict = {'links': dict([(str(k), v) for (k, v) in self.iteritems()]),
-                      'parent_links': dict([(str(k), v) for (k, v) in self.parent_links.iteritems()]),
-                      'nodes': self.nodes}
+            m_dict = {
+                'links': dict([(str(k), v) for (k, v) in self.iteritems()]),
+                'parent_links': dict(
+                    [(str(k), v) for (k, v) in self.parent_links.iteritems()]),
+                'nodes': self.nodes}
             return m_dict
 
         @classmethod
         def from_dict(cls, m_dict):
-            m_dict = dict([(int(k), v) for (k, v) in m_dict.iteritems()])
             return Workflow.Links(m_dict)
 
-    def __init__(self, fireworks, links_dict=None, metadata=None):
+    def __init__(self, fireworks, links_dict=None, name=None, metadata=None):
         """
         :param fireworks: ([FireWork]) - all FireWorks in this workflow
-        :param links_dict: (dict) links between the FWs as (parent_id):[(child_id1, child_id2)]
+        :param links_dict: (dict) links between the FWs as (parent_id):[(
+        child_id1, child_id2)]
         :param metadata: (dict) metadata for this Workflow
         """
 
+        name = 'unnamed WF' if not name else name  # do it this way to prevent None names
+
         links_dict = links_dict if links_dict else {}
 
-        self.id_fw = {}  # main dict containing mapping of an id to a FireWork object
+        self.id_fw = {}  # main dict containing mapping of an id to a
+        # FireWork object
         for fw in fireworks:
             if fw.fw_id in self.id_fw:
                 raise ValueError('FW ids must be unique!')
@@ -422,15 +505,13 @@ class Workflow(FWSerializable):
             if fw.fw_id not in links_dict:
                 links_dict[fw.fw_id] = []
 
-            # transform any non-iterable values to iterables
-            for k, v in links_dict.iteritems():
-                if not isinstance(v, list):
-                    links_dict[k] = [v]
-
         self.links = Workflow.Links(links_dict)
 
-        # sanity: make sure the set of nodes from the links_dict is equal to the set of nodes from id_fw
-        if set(self.links.nodes) != set(self.id_fw.keys()):
+        self.name = name
+
+        # sanity: make sure the set of nodes from the links_dict is equal to
+        # the set of nodes from id_fw
+        if set(self.links.nodes) != set([int(k) for k in self.id_fw.keys()]):
             raise ValueError("Specified links don't match given FW")
 
         self.metadata = metadata if metadata else {}
@@ -439,6 +520,24 @@ class Workflow(FWSerializable):
     def fws(self):
         return self.id_fw.values()
 
+    @property
+    def state(self):
+
+        # get state of workflow
+        m_state = 'READY'
+        states = [fw.state for fw in self.fws]
+        if all([s == 'COMPLETED' or s == 'CANCELED' for s in states]):
+            m_state = 'COMPLETED'
+        elif any([s == 'DEFUSED' for s in states]):
+            m_state = 'DEFUSED'
+        elif any([s == 'FIZZLED' for s in states]):
+            m_state = 'FIZZLED'
+        elif any([s == 'COMPLETED' for s in states]) or any(
+                [s == 'RUNNING' for s in states]):
+            m_state = 'RUNNING'
+
+        return m_state
+
     def apply_action(self, action, fw_id):
         """
         Apply a FWAction on a FireWork in the Workflow
@@ -496,7 +595,8 @@ class Workflow(FWSerializable):
 
         # re-run all the children
         for child_id in self.links[fw_id]:
-            updated_ids = updated_ids.union(self.rerun_fw(child_id, updated_ids))
+            updated_ids = updated_ids.union(
+                self.rerun_fw(child_id, updated_ids))
 
         # refresh the WF to get the states updated
         return self.refresh(fw_id, updated_ids)
@@ -507,7 +607,8 @@ class Workflow(FWSerializable):
 
         :param wf: New Workflow to add
         :param fw_id: id of the FireWork on which to add the Workflow
-        :param detour: whether to add the children of the current FireWork to the Workflow's leaves
+        :param detour: whether to add the children of the current FireWork to
+         the Workflow's leaves
         :return: ([int]) list of FireWork ids that were updated or new
         """
         updated_ids = []
@@ -517,12 +618,16 @@ class Workflow(FWSerializable):
 
         for new_fw in wf.fws:
             if new_fw.fw_id > 0:
-                raise ValueError('FireWorks to add must use a negative fw_id! Got fw_id: {}'.format(new_fw.fw_id))
+                raise ValueError(
+                    'FireWorks to add must use a negative fw_id! Got fw_id: '
+                    '{}'.format(
+                        new_fw.fw_id))
 
             self.id_fw[new_fw.fw_id] = new_fw  # add new_fw to id_fw
 
             if new_fw.fw_id in leaf_ids and detour:
-                self.links[new_fw.fw_id] = self.links[fw_id]  # add children of current FW to new FW
+                self.links[new_fw.fw_id] = self.links[
+                    fw_id]  # add children of current FW to new FW
             elif new_fw.fw_id in leaf_ids:
                 self.links[new_fw.fw_id] = []
             else:
@@ -543,7 +648,8 @@ class Workflow(FWSerializable):
         :return: ([int]) list of FireWork ids that were updated
         """
 
-        updated_ids = updated_ids if updated_ids else set()  # these are the fw_ids to re-enter into the database
+        updated_ids = updated_ids if updated_ids else set()  # these are the
+        # fw_ids to re-enter into the database
 
         fw = self.id_fw[fw_id]
         prev_state = fw.state
@@ -553,22 +659,26 @@ class Workflow(FWSerializable):
             return updated_ids
 
         # what are the parent states?
-        parent_states = [self.id_fw[p].state for p in self.links.parent_links.get(fw_id, [])]
+        parent_states = [self.id_fw[p].state for p in
+                         self.links.parent_links.get(fw_id, [])]
 
         completed_parent_states = ['COMPLETED']
         if fw.spec.get('_allow_fizzled_parents', False):
             completed_parent_states.append('FIZZLED')
 
-        if len(parent_states) != 0 and not all([s in completed_parent_states for s in parent_states]):
+        if len(parent_states) != 0 and not all(
+                [s in completed_parent_states for s in parent_states]):
             m_state = 'WAITING'
 
         else:
-            # my state depends on launch whose state has the highest 'score' in STATE_RANKS
+            # my state depends on launch whose state has the highest 'score'
+            # in STATE_RANKS
             max_score = 0
             m_state = 'READY'
             m_action = None
 
-            # TODO: pick the first launch in terms of end date that matches 'COMPLETED'; multiple might exist
+            # TODO: pick the first launch in terms of end date that matches
+            # 'COMPLETED'; multiple might exist
             for l in fw.launches:
                 if FireWork.STATE_RANKS[l.state] > max_score:
                     max_score = FireWork.STATE_RANKS[l.state]
@@ -582,11 +692,16 @@ class Workflow(FWSerializable):
             updated_ids.add(fw_id)
 
             if m_state == 'COMPLETED':
-                updated_ids = updated_ids.union(self.apply_action(m_action, fw.fw_id))  # apply action on children
+                updated_ids = updated_ids.union(self.apply_action(m_action,
+                                                                  fw.fw_id))
+                # apply
+                # action
+                # on children
 
             # refresh all the children
             for child_id in self.links[fw_id]:
-                updated_ids = updated_ids.union(self.refresh(child_id, updated_ids))
+                updated_ids = updated_ids.union(
+                    self.refresh(child_id, updated_ids))
 
         return updated_ids
 
@@ -633,23 +748,42 @@ class Workflow(FWSerializable):
         # update the Links
         new_l = {}
         for (parent, children) in self.links.iteritems():
-            new_l[old_new.get(parent, parent)] = [old_new.get(child, child) for child in children]
+            new_l[old_new.get(parent, parent)] = [old_new.get(child, child) for
+                                                  child in children]
         self.links = Workflow.Links(new_l)
 
     def to_dict(self):
-        return {'fws': [f.to_dict() for f in self.id_fw.itervalues()], 'links': self.links.to_dict(),
+        return {'fws': [f.to_dict() for f in self.id_fw.itervalues()],
+                'links': self.links.to_dict(),
+                'name': self.name,
                 'metadata': self.metadata}
 
     def to_db_dict(self):
         m_dict = self.links.to_db_dict()
         m_dict['metadata'] = self.metadata
+        m_dict['state'] = self.state
+        m_dict['name'] = self.name
+        return m_dict
+
+    def to_display_dict(self):
+        m_dict = self.to_db_dict()
+        m_dict['states'] = dict([(self.id_fw[x].name + '--' + str(x), self.id_fw[x].state) for x in
+                            m_dict['nodes']])
+        m_dict['nodes'] = [self.id_fw[x].name for x in m_dict['nodes']]
+        m_dict['links'] = dict(
+            [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
+             m_dict['links'].iteritems()])
+        m_dict['parent_links'] = dict(
+            [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
+             m_dict['parent_links'].iteritems()])
         return m_dict
 
     @classmethod
     def from_dict(cls, m_dict):
         # accept either a Workflow dict or a FireWork dict
         if 'fws' in m_dict:
-            return Workflow([FireWork.from_dict(f) for f in m_dict['fws']], Workflow.Links.from_dict(m_dict['links']),
+            return Workflow([FireWork.from_dict(f) for f in m_dict['fws']],
+                            Workflow.Links.from_dict(m_dict['links']), m_dict.get('name'),
                             m_dict['metadata'])
         else:
             return Workflow.from_FireWork(FireWork.from_dict(m_dict))
