@@ -204,9 +204,8 @@ class LaunchPad(FWSerializable):
         fws = []
         for fw_id in links_dict['nodes']:
             fws.append(self.get_fw_by_id(fw_id))
-        links = Workflow.Links.from_dict(links_dict['links']).to_dict()  # necessary because Mongo no like int keys
 
-        return Workflow(fws, links, links_dict['metadata'])
+        return Workflow(fws, links_dict['links'], links_dict['name'], links_dict['metadata'])
 
     def get_fw_ids(self, query=None, sort=False):
         """
@@ -221,6 +220,18 @@ class LaunchPad(FWSerializable):
 
         return fw_ids
 
+    def get_wf_ids(self, query=None):
+        """
+        Return one fw id for all workflows that match a query,
+        :param query: a dict representing a Mongo query
+        """
+        fw_ids = []
+        criteria = query if query else {}
+        for fw in self.workflows.find(criteria, {"nodes": True}):
+            fw_ids.append(fw["nodes"][0])
+
+        return fw_ids
+
     def run_exists(self, fworker=None):
         """
         Checks to see if the database contains any FireWorks that are ready to run
