@@ -5,13 +5,11 @@ TODO: add docs!
 """
 
 import os
-import shlex
 import subprocess
 import getpass
-import threading
-import traceback
+import re
 from fireworks.queue.queue_adapter import QueueAdapterBase
-from fireworks.utilities.fw_utilities import log_fancy, log_exception
+from fireworks.utilities.fw_utilities import get_fw_logger, log_fancy, log_exception
 
 
 __author__ = 'Anubhav Jain, Michael Kocher'
@@ -22,50 +20,6 @@ __email__ = 'ajain@lbl.gov'
 __date__ = 'Dec 12, 2012'
 
 
-class Command(object):
-    """
-    From https://gist.github.com/kirpit/1306188
-
-    Enables to run subprocess commands in a different thread with TIMEOUT option.
-
-    Based on jcollado's solution:
-    http://stackoverflow.com/questions/1191374/subprocess-with-timeout/4825933#4825933
-    """
-    command = None
-    process = None
-    status = None
-    output, error = '', ''
-
-    def __init__(self, command):
-        if isinstance(command, basestring):
-            command = shlex.split(command)
-        self.command = command
-
-    def run(self, timeout=None, **kwargs):
-        """ Run a command then return: (status, output, error). """
-        def target(**kwargs):
-            try:
-                self.process = subprocess.Popen(self.command, **kwargs)
-                self.output, self.error = self.process.communicate()
-                self.status = self.process.returncode
-            except:
-                self.error = traceback.format_exc()
-                self.status = -1
-        # default stdout and stderr
-        if 'stdout' not in kwargs:
-            kwargs['stdout'] = subprocess.PIPE
-        if 'stderr' not in kwargs:
-            kwargs['stderr'] = subprocess.PIPE
-        # thread
-        thread = threading.Thread(target=target, kwargs=kwargs)
-        thread.start()
-        thread.join(timeout)
-        if thread.is_alive():
-            self.process.terminate()
-            thread.join()
-        return self.status, self.output, self.error
-
-
 class PBSAdapterNERSC(QueueAdapterBase):
     _fw_name = 'PBSAdapter (NERSC)'
     template_file = os.path.join(os.path.dirname(__file__), 'PBS_template.txt')
@@ -97,11 +51,9 @@ class PBSAdapterNERSC(QueueAdapterBase):
 
             else:
                 # some qsub error, e.g. maybe wrong queue specified, don't have permission to submit, etc...
-                msgs = [
-                    'Error in job submission with PBS file {f} and cmd {c}'.format(f=script_file,
-                                                                                   c=cmd),
-                    'The error response reads: {}'.format(p.stderr.read())]
-                log_fancy(pbs_logger, msgs, 'error')
+                msgs = ['Error in job submission with PBS file {f} and cmd {c}'.format(f=script_file, c=cmd),
+                        'The error response reads: {}'.format(p.stderr.read())]
+                log_fancy(pbs_logger, 'error', msgs)
 
         except:
             # random error, e.g. no qsub on machine!
@@ -115,23 +67,24 @@ class PBSAdapterNERSC(QueueAdapterBase):
             username = getpass.getuser()
 
         # run qstat
-        qstat = Command(['qstat', '-a', '-u', username])
-        p = qstat.run(timeout=5)
+        cmd = ['qstat', '-a', '-u', username]
+        p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE)
+        p.wait()
 
         # parse the result
-        if p[0] == 0:
+        if p.returncode == 0:
             # lines should have this form
             # '1339044.sdb          username  queuename    2012-02-29-16-43  20460   --   --    --  00:20 C 00:09'
             # count lines that include the username in it
 
             # TODO: only count running or queued jobs. or rather, *don't* count jobs that are 'C'.
-            outs = p[1].split('\n')
+            outs = p.stdout.readlines()
             njobs = len([line.split() for line in outs if username in line])
             pbs_logger.info('The number of jobs currently in the queue is: {}'.format(njobs))
             return njobs
 
         # there's a problem talking to qstat server?
         msgs = ['Error trying to get the number of jobs in the queue using qstat service',
-                'The error response reads: {}'.format(p[2])]
-        log_fancy(pbs_logger, msgs, 'error')
+                'The error response reads: {}'.format(p.stderr.read())]
+        log_fancy(pbs_logger, 'error', msgs)
         return None
\ No newline at end of file
